/*
 *  Copyright (C) 1995, 1996 Microsoft Corporation. All Rights Reserved.
 *
 *  File: rmmain.cpp
 *
 *  Each of the Direct3D retained mode (D3DRM) samples must be linked with
 *  this file.  It contains the code which allows them to run in the Windows
 *  environment.
 *
 *  A window is created using the rmmain.res which allows the user to select
 *  the Direct3D driver to use and change the render options.
 *
 *  Individual samples are executed through two functions, BuildScene and
 *  OverrideDefaults, as described in rmdemo.h.  Samples can also read
 *  mouse input via ReadMouse.
 */

#define INITGUID
#include <math.h>
#include <windows.h>
#include <stdio.h>
#include  <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <direct.h>
#include <d3drmwin.h>
#include <process.h>
#include <mmsystem.h>

#include "rmdemo.h"             /* prototypes for functions to commumicate with each sample */
#include "rmmain.h"             /* defines constants used in rmmain.rc */
#include "rmerror.h"            /* prototypes for error reporting: error.c */


#define TIMER_ID        1
#define TIMER_RATE      50
int xxx=0;
#define MAX_DRIVERS 5           /* maximum D3D drivers we ever expect to find */

/* 
 * GLOBAL VARIABLES
 */
bool klavrot;
    
    LPDIRECT3DRMMESHBUILDER artifact_builder=NULL;
    LPDIRECT3DRMMESH artifact_mesh=NULL;
    LPDIRECT3DRMFRAME artifact_frame=NULL;

	bool fire=false;
	LPDIRECT3DRMMESHBUILDER bullet_builder=NULL;
    LPDIRECT3DRMMESH bullet_mesh=NULL;
    LPDIRECT3DRMFRAME bullet_frame=NULL;

	LPDIRECT3DRMFRAME  light_frame=NULL;

LPDIRECT3DRMMESHBUILDER light_builder=NULL;
LPDIRECT3DRMMESH light_mesh=NULL;
LPDIRECT3DRMLIGHT light1=NULL;
LPDIRECT3DRMFRAME lights=NULL;


	

	bool activated_enemy=false;
    HRESULT rval;
	struct HEROINFO{
		bool can_fire;
		BYTE life;
		unsigned int fire_damage;
	}HERO;
	const MAX_ENEMY=10;
	struct ENEMY
	{
		D3DVALUE x,y;
		unsigned int life;
		LPDIRECT3DRMMESHBUILDER enemy_builder;
		LPDIRECT3DRMMESH enemy_mesh;
		LPDIRECT3DRMFRAME enemy_frame;
	}ENEMIES[MAX_ENEMY];
	bool Enemy_Already_Created=false;

extern LPDIRECT3DRMFRAME face_frame2;
LPDIRECT3DRM lpD3DRM;           /* Direct3DRM object */
LPDIRECTDRAWCLIPPER lpDDClipper;/* DirectDrawClipper object */

struct _myglobs {
    LPDIRECT3DRMDEVICE dev;     /* Direct3DRM device */
    LPDIRECT3DRMVIEWPORT view;  /* Direct3DRM viewport through which we view
                                   the scene */
    LPDIRECT3DRMFRAME scene;    /* Master frame in which others are placed */
    LPDIRECT3DRMFRAME camera;   /* Frame describing the users POV */

    GUID DriverGUID[MAX_DRIVERS];     /* GUIDs of the available D3D drivers */
    char DriverName[MAX_DRIVERS][50]; /* names of the available D3D drivers */
    int  NumDrivers;                  /* number of available D3D drivers */
    int  CurrDriver;                  /* number of D3D driver currently
                                         being used */

    D3DRMRENDERQUALITY RenderQuality;   /* current shade mode, fill mode and
                                           lighting state */
    D3DRMTEXTUREQUALITY TextureQuality; /* current texture interpolation */
    BOOL bDithering;                    /* is dithering on? */
    BOOL bAntialiasing;                 /* is antialiasing on? */

    BOOL bQuit;                 /* program is about to terminate */
    BOOL bInitialized;          /* all D3DRM objects have been initialized */
    BOOL bMinimized;            /* window is minimized */
    BOOL bSingleStepMode;       /* render one frame at a time */
    BOOL bDrawAFrame;           /* render on this pass of the main loop */
    BOOL bNoTextures;           /* this sample doesn't use any textures */
    BOOL bConstRenderQuality;   /* this sample is not constructed with
                                   MeshBuilders and so the RenderQuality
                                   cannot be changed */

    int BPP;                    /* bit depth of the current display mode */

    int mouse_buttons;          /* mouse button state */
    int mouse_x;                /* mouse cursor x position */
    int mouse_y;                /* mouse cursor y position */
} myglobs;

/*
 * PROTOTYPES
 */
static HWND InitApp(HINSTANCE, int);
static void InitGlobals(void);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);
static HRESULT CreateDevAndView(LPDIRECTDRAWCLIPPER lpDDClipper, int driver, int width, int height);
static BOOL RenderLoop(void);
static void CleanUpAndPostQuit(void);
static HRESULT SetRenderState(void);
static BOOL EnumDevices(HWND win);
extern "C" void ReadMouse(int*, int*, int*);

/****************************************************************************/
/*                               WinMain                                    */
/****************************************************************************/
/*
 * Initializes the application then enters a message loop which renders the
 * scene until a quit message is received.
 */
int PASCAL
WinMain (HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline, int cmdshow)
{
    HWND    hwnd;
    MSG     msg;
    HACCEL  accel;
    int     failcount = 0;  /* number of times RenderLoop has failed */
	klavrot=false;
if (!joyGetNumDevs())
    {
        MessageBox(NULL, "There are no joystick devices installed. Exiting.",
                   NULL, MB_OK | MB_ICONEXCLAMATION);
        return FALSE;
    }

    prev_inst;
    cmdline;

    /*
     * Create the window and initialize all objects needed to begin rendering
     */
    if (!(hwnd = InitApp(this_inst, cmdshow)))
        return 1;

    accel = LoadAccelerators(this_inst, "AppAccel");

    while (!myglobs.bQuit) {
        /* 
         * Monitor the message queue until there are no pressing
         * messages
         */

        while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
            if (msg.message == WM_QUIT) {
                CleanUpAndPostQuit();
                break;
            }
            if (!TranslateAccelerator(msg.hwnd, accel, &msg)) {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            }
        }
	if (myglobs.bQuit)
	    break;
        /* 
         * If the app is not minimized, not about to quit and D3DRM has
         * been initialized, we can render
         */
        if (!myglobs.bMinimized && !myglobs.bQuit && myglobs.bInitialized) {
            /*
             * If were are not in single step mode or if we are and the
             * bDrawAFrame flag is set, render one frame
             */
            if (!(myglobs.bSingleStepMode && !myglobs.bDrawAFrame)) {
                /* 
                 * Attempt to render a frame, if it fails, take a note.  If
                 * rendering fails more than twice, abort execution.
                 */
                if (!RenderLoop())
                    ++failcount;
                if (failcount > 2) {
                    Msg("Rendering has failed too many times.  Aborting execution.\n");
                    CleanUpAndPostQuit();
                    break;
                }
            }
            /*
             * Reset the bDrawAFrame flag if we are in single step mode
             */
            if (myglobs.bSingleStepMode)
                myglobs.bDrawAFrame = FALSE;
        } else {
	    WaitMessage();
	}
    }
	
    //if (MessageBox(hwnd,"   ,         .","",MB_YESNO))
		//DestroyWindow(hwnd);
	 return msg.wParam;

}

/****************************************************************************/
/*                   Initialization and object creation                     */
/****************************************************************************/
/*
 * InitApp
 * Creates window and initializes all objects neccessary to begin rendering
 */
static HWND
InitApp(HINSTANCE this_inst, int cmdshow)
{
    HWND win;
    HDC hdc;
    DWORD flags;
    WNDCLASS wc;
    Defaults defaults;
    HRESULT rval;
    RECT rc;

    /*
     * set up and registers the window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = this_inst;
    wc.hIcon = LoadIcon(this_inst, "AppIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName = "AppMenu";
    wc.lpszClassName = "D3DRM Example";
    if (!RegisterClass(&wc))
        return FALSE;
    /*
     * Initialize the global variables and allow the sample code to override
     * some of these default settings.
     */
    InitGlobals();
	myglobs.bNoTextures =true;
    defaults.bNoTextures = myglobs.bNoTextures;
    defaults.bConstRenderQuality = myglobs.bConstRenderQuality;
    defaults.bResizingDisabled = FALSE;
    lstrcpy(defaults.Name, "D3DRM Example");
    OverrideDefaults(&defaults);
    myglobs.bNoTextures = defaults.bNoTextures;
    myglobs.bConstRenderQuality = defaults.bConstRenderQuality;
    /*
     * Create the window
     */
    if (defaults.bResizingDisabled)
        flags =  WS_VISIBLE | WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU |
                 WS_MINIMIZEBOX | WS_MAXIMIZEBOX;
    else
        flags = WS_OVERLAPPEDWINDOW;
    win =
        CreateWindow
        (   "D3DRM Example",            /* class */
            defaults.Name,              /* caption */
            flags,                      /* style */
            CW_USEDEFAULT,              /* init. x pos */
            CW_USEDEFAULT,              /* init. y pos */
            800,                        /* init. x size */
            600,                        /* init. y size */
            NULL,                       /* parent window */
            NULL,                       /* menu handle */
            this_inst,                  /* program handle */
            NULL                        /* create parms */
        );
    if (!win)
        return FALSE;

	SetTimer( win, TIMER_ID, TIMER_RATE, NULL ); 
			
    /*
     * Record the current display BPP
     */
    hdc = GetDC(win);
    myglobs.BPP = GetDeviceCaps(hdc, BITSPIXEL);
    ReleaseDC(win, hdc);
    /*
     * Enumerate the D3D drivers and select one
     */
    if (!EnumDevices(win))
        return FALSE;
    /*
     * Create the D3DRM object and the D3DRM window object
     */
    rval = Direct3DRMCreate(&lpD3DRM);
    if (rval != D3DRM_OK) {
        Msg("Failed to create Direct3DRM.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    /*
     * Create the master scene frame and camera frame
     */
    rval = lpD3DRM->CreateFrame(NULL, &myglobs.scene);
    if (rval != D3DRM_OK) {
        Msg("Failed to create the master scene frame.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    rval = lpD3DRM->CreateFrame(myglobs.scene, &myglobs.camera);
    if (rval != D3DRM_OK) {
        Msg("Failed to create the camera's frame.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    rval = myglobs.camera->SetPosition(myglobs.scene, D3DVAL(0.0), D3DVAL(0.0), D3DVAL(0.0));
    if (rval != D3DRM_OK) {
        Msg("Failed to position the camera in the frame.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    /*
     * Create a clipper and associate the window with it
     */
    rval = DirectDrawCreateClipper(0, &lpDDClipper, NULL);
    if (rval != DD_OK) {
        Msg("Failed to create DirectDrawClipper");
        return FALSE;
    }
    rval = lpDDClipper->SetHWnd(0, win);
    if (rval != DD_OK) {
        Msg("Failed to set hwnd on the clipper");
        return FALSE;
    }
    /*
     * Created the D3DRM device with the selected D3D driver
     */
    GetClientRect(win, &rc);
    if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver, rc.right, rc.bottom)) {
	myglobs.CurrDriver = 0;
	if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver, rc.right, rc.bottom)) {
	    Msg("Failed to create the D3DRM device.\n%s", D3DRMErrorToString(rval));
	    return FALSE;
	}
    }
    /*
     * Create the scene to be rendered by calling this sample's BuildScene
     */
    if (!BuildScene(myglobs.dev, myglobs.view, myglobs.scene, myglobs.camera))
        return FALSE;
    /*
     * Display the window
     */
    ShowWindow(win, cmdshow);
    UpdateWindow(win);
    /*
     * Now we are ready to render
     */
    myglobs.bInitialized = TRUE;

    return win;
}

/*
 * CreateDevAndView
 * Create the D3DRM device and viewport with the given D3D driver and of the
 * specified size.
 */
static HRESULT
CreateDevAndView(LPDIRECTDRAWCLIPPER lpDDClipper, int driver, int width, int height)
{
    HRESULT rval;

    if (!width || !height)
        return D3DRMERR_BADVALUE;
    /*
     * Create the D3DRM device from this window and using the specified D3D
     * driver.
     */
    rval = lpD3DRM->CreateDeviceFromClipper(lpDDClipper, &myglobs.DriverGUID[driver],
                                        width, height, &myglobs.dev);
    if (rval != D3DRM_OK)
        return rval;
    /*
     * Create the D3DRM viewport using the camera frame.  Set the background
     * depth to a large number.  The width and height may be slightly
     * adjusted, so get them from the device to be sure.
     */
    width = myglobs.dev->GetWidth();
    height = myglobs.dev->GetHeight();
    rval = lpD3DRM->CreateViewport(myglobs.dev, myglobs.camera, 0, 0, width,
                                   height, &myglobs.view);
    if (rval != D3DRM_OK) {
	myglobs.bInitialized = FALSE;
        RELEASE(myglobs.dev);
        return rval;
    }
    rval = myglobs.view->SetBack(D3DVAL(5000.0));
    if (rval != D3DRM_OK) {
	myglobs.bInitialized = FALSE;
        RELEASE(myglobs.dev);
        RELEASE(myglobs.view);
        return rval;
    }
    /*
     * Set the render quality, fill mode, lighting state and color shade info
     */
    if (rval = SetRenderState()) {
	myglobs.bInitialized = FALSE;
        RELEASE(myglobs.dev);
        RELEASE(myglobs.view);
        return rval;
    }

    return D3DRM_OK;
}

/****************************************************************************/
/*                         D3D Device Enumeration                           */
/****************************************************************************/
/*
 * BPPToDDBD
 * Converts bits per pixel to a DirectDraw bit depth flag
 */
static DWORD
BPPToDDBD(int bpp)
{
    switch(bpp) {
        case 1:
            return DDBD_1;
        case 2:
            return DDBD_2;
        case 4:
            return DDBD_4;
        case 8:
            return DDBD_8;
        case 16:
            return DDBD_16;
        case 24:
            return DDBD_24;
        case 32:
            return DDBD_32;
        default:
            return 0;
    }
}

/*
 * enumDeviceFunc
 * Callback function which records each usable D3D driver's name and GUID
 * Chooses a driver to begin with and sets *lpContext to this starting driver
 */
static HRESULT
WINAPI enumDeviceFunc(LPGUID lpGuid, LPSTR lpDeviceDescription, LPSTR lpDeviceName,
        LPD3DDEVICEDESC lpHWDesc, LPD3DDEVICEDESC lpHELDesc, LPVOID lpContext)
{
    static BOOL hardware = FALSE; /* current start driver is hardware */
    static BOOL mono = FALSE;     /* current start driver is mono light */
    LPD3DDEVICEDESC lpDesc;
    int *lpStartDriver = (int *)lpContext;
    /*
     * Decide which device description we should consult
     */
    lpDesc = lpHWDesc->dcmColorModel ? lpHWDesc : lpHELDesc;
    /*
     * If this driver cannot render in the current display bit depth skip
     * it and continue with the enumeration.
     */
    if (!(lpDesc->dwDeviceRenderBitDepth & BPPToDDBD(myglobs.BPP)))
        return D3DENUMRET_OK;
    /*
     * Record this driver's info
     */
    memcpy(&myglobs.DriverGUID[myglobs.NumDrivers], lpGuid, sizeof(GUID));
    lstrcpy(&myglobs.DriverName[myglobs.NumDrivers][0], lpDeviceName);
    /*
     * Choose hardware over software, RGB lights over mono lights
     */
    if (*lpStartDriver == -1) {
        /*
         * this is the first valid driver
         */
        *lpStartDriver = myglobs.NumDrivers;
        hardware = lpDesc == lpHWDesc ? TRUE : FALSE;
        mono = lpDesc->dcmColorModel & D3DCOLOR_MONO ? TRUE : FALSE;
    } else if (lpDesc == lpHWDesc && !hardware) {
        /*
         * this driver is hardware and start driver is not
         */
        *lpStartDriver = myglobs.NumDrivers;
        hardware = lpDesc == lpHWDesc ? TRUE : FALSE;
        mono = lpDesc->dcmColorModel & D3DCOLOR_MONO ? TRUE : FALSE;
    } else if ((lpDesc == lpHWDesc && hardware ) || (lpDesc == lpHELDesc
                                                     && !hardware)) {
        if (lpDesc->dcmColorModel == D3DCOLOR_MONO && !mono) {
            /*
             * this driver and start driver are the same type and this
             * driver is mono while start driver is not
             */
            *lpStartDriver = myglobs.NumDrivers;
            hardware = lpDesc == lpHWDesc ? TRUE : FALSE;
            mono = lpDesc->dcmColorModel & D3DCOLOR_MONO ? TRUE : FALSE;
        }
    }
    myglobs.NumDrivers++;
    if (myglobs.NumDrivers == MAX_DRIVERS)
        return (D3DENUMRET_CANCEL);
    return (D3DENUMRET_OK);
}

/*
 * EnumDevices
 * Enumerate the available D3D drivers, add them to the file menu, and choose
 * one to use.
 */
static BOOL
EnumDevices(HWND win)
{
    LPDIRECTDRAW lpDD;
    LPDIRECT3D lpD3D;
    HRESULT rval;
    HMENU hmenu;
    int i;

    /*
     * Create a DirectDraw object and query for the Direct3D interface to use
     * to enumerate the drivers.
     */
    rval = DirectDrawCreate(NULL, &lpDD, NULL);
    if (rval != DD_OK) {
        Msg("Creation of DirectDraw HEL failed.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    rval = lpDD->QueryInterface(IID_IDirect3D, (void**) &lpD3D);
    if (rval != DD_OK) {
        Msg("Creation of Direct3D interface failed.\n%s", D3DRMErrorToString(rval));
        lpDD->Release();
        return FALSE;
    }
    /*
     * Enumerate the drivers, setting CurrDriver to -1 to initialize the
     * driver selection code in enumDeviceFunc
     */
    myglobs.CurrDriver = -1;
    rval = lpD3D->EnumDevices(enumDeviceFunc, &myglobs.CurrDriver);
    if (rval != DD_OK) {
        Msg("Enumeration of drivers failed.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    /*
     * Make sure we found at least one valid driver
     */
    if (myglobs.NumDrivers == 0) {
        Msg("Could not find a D3D driver which is compatible with this display depth");
        return FALSE;
    }
    lpD3D->Release();
    lpDD->Release();
    /*
     * Add the driver names to the File menu
     */
    hmenu = GetSubMenu(GetMenu(win), 0);
    for (i = 0; i < myglobs.NumDrivers; i++) {
        InsertMenu(hmenu, 5 + i, MF_BYPOSITION | MF_STRING, MENU_FIRST_DRIVER + i,
                   myglobs.DriverName[i]);
    }
    return TRUE;
}

/****************************************************************************/
/*                             Render Loop                                  */
/****************************************************************************/
/*
 * Clear the viewport, render the next frame and update the window
 */
static BOOL
RenderLoop()
{
    HRESULT rval;
    /*
     * Tick the scene
     */
    rval = myglobs.scene->Move(D3DVAL(1.0));
    if (rval != D3DRM_OK) {
        Msg("Moving scene failed.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    /* 
     * Clear the viewport
     */
    rval = myglobs.view->Clear();
    if (rval != D3DRM_OK) {
        Msg("Clearing viewport failed.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    /*
     * Render the scene to the viewport
     */
    rval = myglobs.view->Render(myglobs.scene);
    if (rval != D3DRM_OK) {
        Msg("Rendering scene failed.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    /*
     * Update the window
     */
    rval = myglobs.dev->Update();
    if (rval != D3DRM_OK) {
        Msg("Updating device failed.\n%s", D3DRMErrorToString(rval));
        return FALSE;
    }
    return TRUE;
}


/****************************************************************************/
/*                    Windows Message Handlers                              */
/****************************************************************************/
/*
 * AppAbout
 * About box message handler
 */
BOOL
FAR PASCAL AppAbout(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_COMMAND:
      if (LOWORD(wparam) == IDOK)
        EndDialog(hwnd, TRUE);
      break;

    case WM_INITDIALOG:
      return TRUE;
  }
  return FALSE;
}


/*
 * WindowProc
 * Main window message handler
 */

HRESULT CreateRectangle(long x, long y, long z)
{
						//myglobs.camera->GetOrientation(myglobs.scene,&dir,&up);
	LPDIRECT3DRMMESHBUILDER bullet_builder=NULL;
    LPDIRECT3DRMMESH bullet_mesh=NULL;
	LPDIRECT3DRMMESHBUILDER builder=NULL;
    LPDIRECT3DRMFRAME bullet_frame=NULL;
	//     LPDIRECT3DRMTEXTURE tex; tex= NULL;
    LPDIRECT3DRMMATERIAL mat = NULL;
 
    LPDIRECT3DRMLIGHT l1 = NULL;
    LPDIRECT3DRMLIGHT l2 = NULL;
    D3DRMBOX box;
    D3DVALUE miny, maxy;
    D3DVALUE height;
builder=NULL;
		HRESULT rval;
		if (FAILED(lpD3DRM->CreateMeshBuilder(&bullet_builder))) MessageBox(NULL,"error","error",MB_OK);
	//if (!build_face2(bullet_builder));

	#define VS 1
    int vertex_count = 4;
    D3DVECTOR vertices[4] = {D3DVAL(VS),D3DVAL(0),  D3DVAL(VS),
	-D3DVAL(VS), D3DVAL(0), D3DVAL(VS),
	-D3DVAL(VS), D3DVAL(0), -D3DVAL(VS),
	D3DVAL(VS),  D3DVAL(0), -D3DVAL(VS)
    };
    int normal_count = 0;
    D3DVECTOR normals[]= {
	//D3DVAL(0.5), D3DVAL(0.7), D3DVAL(0.5),
	//D3DVAL(0.5), D3DVAL(0.7), -D3DVAL(0.5),
	//-D3DVAL(0.5), D3DVAL(0.7), -D3DVAL(0.5),
	-D3DVAL(0.5), D3DVAL(0.7), D3DVAL(0.5)
    };
    unsigned long face_data[] = {//3, 0, 1, 2
	//3, 2, 1,0,
	//3, 0, 2, 3,
	//3, 3, 2, 0
	4,3,2,1,0,
	//4,0,1,2,3
	};

	//if (FAILED(lpD3DRM->LoadTexture( "tex7.ppm", &tex)))  goto generic_error;
	    //if (rval != D3DRM_OK) {
    //    Msg("Failed to load tex7.ppm.\n%s", D3DRMErrorToString(rval));
	//goto ret_with_error;
    //}
    //if (FAILED(tex->SetColors( 32)))
	//goto generic_error;
    //if (FAILED(tex->SetDecalScale( TRUE)))
	//goto generic_error;
    //if (FAILED(tex->SetDecalOrigin( 128, 128)))


     if (FAILED(lpD3DRM->CreateMaterial( D3DVAL(0.0), &mat)))
	goto generic_error;

	
    //if (FAILED(bullet_builder->Scale( D3DVAL(2), D3DVAL(2), D3DVAL(2))))
	//goto generic_error;
	//myglobs.scene->SetSceneBackgroundRGB(D3DVAL(1),D3DVAL(100),D3DVAL(1));
//	bullet_builder->SetTextureCoordinates(0,D3DVAL(0.1),D3DVAL(0.1));
   //		bullet_mesh->SetGroupTexture(
	//if (FAILED(bullet_builder->SetTexture(tex)))
	//goto generic_error;
  //  if (FAILED(bullet_builder->SetMaterial(mat)))
	//goto generic_error;
  //  if (FAILED(bullet_builder->SetColorRGB( D3DVAL(1.0), D3DVAL(1.0), D3DVAL(1.0))))
	//goto generic_error;
    bullet_builder->AddFaces( vertex_count, vertices, normal_count, 
                          normals, face_data,NULL);
	//bullet_builder->Load( "SPHERE4.x",NULL,
    //			D3DRMLOAD_FROMFILE, NULL, NULL);

 	if (FAILED(lpD3DRM->CreateFrame(myglobs.scene, &bullet_frame))) MessageBox(NULL,"error","error",MB_OK);
	//tex->Release;
	//tex->SetColors(20);
	lpD3DRM->CreateMeshBuilder(&builder);
//	myglobs.bNoTextures=true;
	//  if (FAILED(bullet_builder->SetTexture( tex)))
	//goto generic_error;
    if (FAILED(bullet_builder->SetMaterial( mat)))
	goto generic_error;
   // if (FAILED(bullet_builder->SetColorRGB(D3DVAL(1.0), D3DVAL(1.0), D3DVAL(1.0))))
	//goto generic_error;
	//	bullet_builder->SetColorRGB(D3DVAL(100),D3DVAL(100),D3DVAL(100));
	//bullet_builder->SetMaterial(mat);
	//bullet_builder->Scale(D3DVAL(10),D3DVAL(10),D3DVAL(10));
//	bullet_frame->SetRotation(myglobs.scene,D3DVAL(1),D3DVAL(1),D3DVAL(1),D3DVAL(0.01));
	if (FAILED(bullet_builder->CreateMesh(&bullet_mesh))) MessageBox(NULL,"error","error",MB_OK);
RELEASE(builder);
	



//bullet_builder->SetQuality(D3DRMSHADE_FLAT);
	RELEASE(bullet_builder);

	if (FAILED(bullet_frame->AddVisual((LPDIRECT3DRMVISUAL) bullet_mesh))) MessageBox(NULL,"error","error",MB_OK);
    //D3DVAL(up.x), D3DVAL(up.y), D3DVAL(up.z))));					//bullet_frame->SetRotation(bullet_frame,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),0);
					
	RELEASE(bullet_mesh);
	if (FAILED(bullet_frame->SetPosition(myglobs.scene,D3DVAL(x),
                         D3DVAL(y), D3DVAL(z)))) MessageBox(NULL,"error","error",MB_OK);
	RELEASE(bullet_frame);
		
	//	if (FAILED(lpD3DRM->CreateFrame( myglobs.scene, &artifact_frame))) MessageBox(NULL,"error","error",MB_OK);
	//	if (FAILED(lpD3DRM->CreateMeshBuilder(&artifact_builder))) MessageBox(NULL,"error","error",MB_OK);
	//	if (!build_spaceship(artifact_builder));
	//	if (FAILED(artifact_builder->CreateMesh(&artifact_mesh))) MessageBox(NULL,"error","error",MB_OK);
	//	RELEASE(artifact_builder);
	//	if (FAILED(artifact_frame->AddVisual((LPDIRECT3DRMVISUAL) artifact_mesh))) MessageBox(NULL,"error","error",MB_OK);
	//	if (FAILED(artifact_frame->SetPosition(myglobs.scene, D3DVAL(2.5), 
      //                   D3DVAL(1.0), D3DVAL(-13)))) MessageBox(NULL,"error","error",MB_OK);
	//	if (FAILED(artifact_frame->SetOrientation(myglobs.scene, D3DVAL(0.0), D3DVAL(1), D3DVAL(0.0),
      //                     D3DVAL(2.0), D3DVAL(1.0), -D3DVAL(1.0))));
//		RELEASE(artifact_mesh);
//		RELEASE(artifact_frame);
		return rval;

generic_error:
		MessageBox(NULL,"   .","ERROR",MB_OK);
					
}
	


void FireButtonPressed(HWND win)
{
						if (!fire)
					{
					
					
					//D3DVECTOR dir;
					//D3DVECTOR up;
					//myglobs.camera->GetOrientation(myglobs.scene,&dir,&up);
					if (FAILED(lpD3DRM->CreateFrame( myglobs.scene, &bullet_frame))) MessageBox(win,"error","error",MB_OK);
					if (FAILED(lpD3DRM->CreateMeshBuilder(&bullet_builder))) MessageBox(win,"error","error",MB_OK);
					if (!build_face1(bullet_builder));
					if (FAILED(bullet_builder->CreateMesh(&bullet_mesh))) MessageBox(win,"error","error",MB_OK);
					//RELEASE(bullet_builder);
					if (FAILED(bullet_frame->AddVisual((LPDIRECT3DRMVISUAL) bullet_mesh))) MessageBox(win,"error","error",MB_OK);
					
//					if (FAILED(bullet_frame->SetOrientation(myglobs.scene, D3DVAL(0), D3DVAL(dir.x), D3DVAL(dir.y),
  //                                     D3DVAL(2.0), D3DVAL(1.0), -D3DVAL(1.0))));
					//if (FAILED(bullet_frame->SetOrientation(myglobs.scene, D3DVAL(dir.x), D3DVAL(dir.y), D3DVAL(dir.z),
                    //                   D3DVAL(up.x), D3DVAL(up.y), D3DVAL(up.z))));					//bullet_frame->SetRotation(bullet_frame,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),0);
									
					//RELEASE(bullet_mesh);
					//RELEASE(bullet_frame);
					
					}
					
						if (FAILED(bullet_frame->SetPosition(myglobs.camera,-D3DVAL(0.25), 
                                     -D3DVAL(0.25), -D3DVAL(0.25)))) MessageBox(win,"error","error",MB_OK);
					//if (FAILED(bullet_frame->SetOrientation(myglobs.scene, D3DVAL(0.0), D3DVAL(1), D3DVAL(0.0),
                    //                    D3DVAL(2.0), D3DVAL(1.0), -D3DVAL(1.0))));
					//bullet_frame->SetRotation(bullet_frame,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),0);
					
					D3DVECTOR dir;
					D3DVECTOR up;
					myglobs.camera->GetOrientation(myglobs.scene,&dir,&up);
					if (FAILED(bullet_frame->SetOrientation(myglobs.scene, D3DVAL(dir.x), D3DVAL(dir.y), D3DVAL(dir.z),
                                       D3DVAL(up.x), D3DVAL(up.y), D3DVAL(up.z))));		


/////////////////////////////////////////////////////////

if (!fire)
{

    if (FAILED(lpD3DRM->CreateFrame(myglobs.scene, &light_frame)));
    if (FAILED(lpD3DRM->CreateFrame( light_frame, &lights)));
if (FAILED(lpD3DRM->CreateLightRGB(D3DRMLIGHT_POINT, D3DVAL(0),
     D3DVAL(0), D3DVAL(3), &light1)));

if (FAILED(lights->AddLight(light1)));

//RELEASE(light1);
if (FAILED(lpD3DRM->CreateMeshBuilder( &light_builder)));

//HRESULT rval1;
//rval1 = 

light_builder->Load("sphere0.x", NULL,
    				D3DRMLOAD_FROMFILE, NULL, NULL);
  //  if (rval1 != D3DRM_OK) {
     //   Msg("Failed to load sphere0.x\n");

  if (FAILED(light_builder->SetColorRGB( D3DVAL(1), D3DVAL(1),
                                        D3DVAL(200))));

    if (FAILED(light_builder->Scale( D3DVAL(0.1), D3DVAL(0.1), 
                                 D3DVAL(0.1))));

if (FAILED(light_builder->SetQuality( D3DRMRENDER_UNLITFLAT)));

if (FAILED(light_builder->CreateMesh( &light_mesh)));

//RELEASE(light_builder);

 if (FAILED(lights->AddVisual( (LPDIRECT3DRMVISUAL) light_mesh)));

  //RELEASE(light_mesh);
  // RELEASE(light_frame);

 // RELEASE(lights);
}
light_frame->SetPosition(myglobs.camera,D3DVAL(-0.25),D3DVAL(-0.25),D3DVAL(-0.25));
					if (FAILED(bullet_frame->SetOrientation(light_frame, D3DVAL(dir.x), D3DVAL(dir.y), D3DVAL(dir.z),
                                       D3DVAL(up.x), D3DVAL(up.y), D3DVAL(up.z))));		
fire=true;

//					break;
//				}
//			}
			
			
}






float angle=0.1;







LONG FAR PASCAL WindowProc(HWND win, UINT msg, WPARAM wparam, LPARAM lparam)
{
    int i;
    HRESULT rval;
    RECT rc;
	HDC hdc;
	PAINTSTRUCT ps;
	if (msg==WM_CREATE) 
	{


			//	MessageBox(win,"Set Joystick Capture","",MB_OK);
            /* Capture the joystick. If this fails, beep and display
             * error, then quit.
             */
					srand(10);
            if(joySetCapture(win, JOYSTICKID1, 0, FALSE))
            {
                MessageBeep(MB_ICONEXCLAMATION);
                MessageBox(win, "Couldn't capture the joystick", NULL, 
                           MB_OK | MB_ICONEXCLAMATION);
                //return -1;
            }  

			
	}

		if (!myglobs.bInitialized) {
		return DefWindowProc(win, msg, wparam, lparam);
	}

    switch (msg)   
	{
		case WM_CREATE:
		{
	//	MessageBox(win,"12312412345","",MB_OK);
            /* Capture the joystick. If this fails, beep and display
             * error, then quit.
             */
            if(joySetCapture(win, JOYSTICKID1, 0, FALSE))
            {
                MessageBeep(MB_ICONEXCLAMATION);
                MessageBox(win, "Couldn't capture the joystick", NULL, 
                           MB_OK | MB_ICONEXCLAMATION);
                //return -1;
            }            
            break;
		}
        case WM_LBUTTONDOWN:
        case WM_LBUTTONUP:
        case WM_RBUTTONDOWN:
        case WM_RBUTTONUP:
        case WM_MOUSEMOVE:
            /*
             * Record the mouse state for ReadMouse
             */
            myglobs.mouse_buttons = wparam;
            myglobs.mouse_x = LOWORD(lparam);
            myglobs.mouse_y = HIWORD(lparam);
            break;
        case WM_INITMENUPOPUP:
            /*
             * Check and enable the appropriate menu items
             */
            CheckMenuItem((HMENU)wparam, MENU_STEP,(myglobs.bSingleStepMode) ? MF_CHECKED : MF_UNCHECKED);
            EnableMenuItem((HMENU)wparam, MENU_GO,(myglobs.bSingleStepMode) ? MF_ENABLED : MF_GRAYED);
            if (!myglobs.bConstRenderQuality) {
                CheckMenuItem((HMENU)wparam, MENU_LIGHTING, (myglobs.RenderQuality & D3DRMLIGHT_MASK) == D3DRMLIGHT_ON ? MF_CHECKED : MF_GRAYED);
                CheckMenuItem((HMENU)wparam, MENU_FLAT, (myglobs.RenderQuality & D3DRMSHADE_MASK) == D3DRMSHADE_FLAT ? MF_CHECKED : MF_UNCHECKED);
                CheckMenuItem((HMENU)wparam, MENU_GOURAUD, (myglobs.RenderQuality & D3DRMSHADE_MASK) == D3DRMSHADE_GOURAUD ? MF_CHECKED : MF_UNCHECKED);
                CheckMenuItem((HMENU)wparam, MENU_PHONG, (myglobs.RenderQuality & D3DRMSHADE_MASK) == D3DRMSHADE_PHONG ? MF_CHECKED : MF_UNCHECKED);
                EnableMenuItem((HMENU)wparam, MENU_PHONG, MF_GRAYED);
                CheckMenuItem((HMENU)wparam, MENU_POINT, (myglobs.RenderQuality & D3DRMFILL_MASK) == D3DRMFILL_POINTS ? MF_CHECKED : MF_UNCHECKED);
                CheckMenuItem((HMENU)wparam, MENU_WIREFRAME, (myglobs.RenderQuality & D3DRMFILL_MASK) == D3DRMFILL_WIREFRAME ? MF_CHECKED : MF_UNCHECKED);
                CheckMenuItem((HMENU)wparam, MENU_SOLID, (myglobs.RenderQuality & D3DRMFILL_MASK) == D3DRMFILL_SOLID ? MF_CHECKED : MF_UNCHECKED);
            } else {
                EnableMenuItem((HMENU)wparam, MENU_LIGHTING, MF_GRAYED);
                EnableMenuItem((HMENU)wparam, MENU_FLAT, MF_GRAYED);
                EnableMenuItem((HMENU)wparam, MENU_GOURAUD, MF_GRAYED);
                EnableMenuItem((HMENU)wparam, MENU_PHONG, MF_GRAYED);
                EnableMenuItem((HMENU)wparam, MENU_POINT, MF_GRAYED);
                EnableMenuItem((HMENU)wparam, MENU_WIREFRAME, MF_GRAYED);
                EnableMenuItem((HMENU)wparam, MENU_SOLID, MF_GRAYED);
            }
            if (!myglobs.bNoTextures) {
                CheckMenuItem((HMENU)wparam, MENU_POINT_FILTER, (myglobs.TextureQuality == D3DRMTEXTURE_NEAREST) ? MF_CHECKED : MF_UNCHECKED);
                CheckMenuItem((HMENU)wparam, MENU_LINEAR_FILTER, (myglobs.TextureQuality == D3DRMTEXTURE_LINEAR) ? MF_CHECKED : MF_UNCHECKED);
            } else {
                EnableMenuItem((HMENU)wparam, MENU_POINT_FILTER, MF_GRAYED);
                EnableMenuItem((HMENU)wparam, MENU_LINEAR_FILTER, MF_GRAYED);
            }
            CheckMenuItem((HMENU)wparam, MENU_DITHERING, (myglobs.bDithering) ? MF_CHECKED : MF_UNCHECKED);
            CheckMenuItem((HMENU)wparam, MENU_ANTIALIAS, (myglobs.bAntialiasing) ? MF_CHECKED : MF_UNCHECKED);
            EnableMenuItem((HMENU)wparam, MENU_ANTIALIAS, MF_GRAYED);
            for (i = 0; i < myglobs.NumDrivers; i++) {
                CheckMenuItem((HMENU)wparam, MENU_FIRST_DRIVER + i,
                       (i == myglobs.CurrDriver) ? MF_CHECKED : MF_UNCHECKED);
            }
            break;
        case WM_COMMAND:
            switch(LOWORD(wparam)) {
                case MENU_ABOUT:
                    DialogBox((HINSTANCE)GetWindowLong(win, GWL_HINSTANCE),
                              "AppAbout", win, (DLGPROC)AppAbout);
                    break;
                case MENU_EXIT:
                    CleanUpAndPostQuit();
                    break;
                case MENU_STEP:
                    /*
                     * Begin single step more or draw a frame if in single
                     * step mode
                     */
                    if (!myglobs.bSingleStepMode) {
                        myglobs.bSingleStepMode = TRUE;
                        myglobs.bDrawAFrame = TRUE;
                    } else if (!myglobs.bDrawAFrame) {
                        myglobs.bDrawAFrame = TRUE;
                    }
                    break;
                case MENU_GO:
                    /*
                     * Exit single step mode
                     */
                    myglobs.bSingleStepMode = FALSE;
                    break;
                /*
                 * Lighting toggle
                 */
                case MENU_LIGHTING:
                    myglobs.RenderQuality ^= D3DRMLIGHT_ON;
                    SetRenderState();
                    break;
                /*
                 * Fill mode selection
                 */
                case MENU_POINT:
                    myglobs.RenderQuality = (myglobs.RenderQuality & ~D3DRMFILL_MASK) | D3DRMFILL_POINTS;
                    SetRenderState();
                    break;
                case MENU_WIREFRAME:
                    myglobs.RenderQuality = (myglobs.RenderQuality & ~D3DRMFILL_MASK) | D3DRMFILL_WIREFRAME;
                    SetRenderState();
                    break;
                case MENU_SOLID:
                    myglobs.RenderQuality = (myglobs.RenderQuality & ~D3DRMFILL_MASK) | D3DRMFILL_SOLID;
                    SetRenderState();
                    break;
                /*
                 * Shade mode selection
                 */
                case MENU_FLAT:
                    myglobs.RenderQuality = (myglobs.RenderQuality & ~D3DRMSHADE_MASK) | D3DRMSHADE_FLAT;
                    SetRenderState();
                    break;
                case MENU_GOURAUD:
                    myglobs.RenderQuality = (myglobs.RenderQuality & ~D3DRMSHADE_MASK) | D3DRMSHADE_GOURAUD;
                    SetRenderState();
                    break;
                case MENU_PHONG:
                    myglobs.RenderQuality = (myglobs.RenderQuality & ~D3DRMSHADE_MASK) | D3DRMSHADE_PHONG;
                    SetRenderState();
                    break;

                case MENU_DITHERING:
                    myglobs.bDithering = !myglobs.bDithering;
                    SetRenderState();
                    break;
                case MENU_ANTIALIAS:
                    myglobs.bAntialiasing = !myglobs.bAntialiasing;
                    break;
                /*
                 * Texture filter selection
                 */
                case MENU_POINT_FILTER:
                    if (myglobs.TextureQuality == D3DRMTEXTURE_NEAREST)
                        break;
                    myglobs.TextureQuality = D3DRMTEXTURE_NEAREST;
                    SetRenderState();
                    break;
                case MENU_LINEAR_FILTER:
                    if (myglobs.TextureQuality == D3DRMTEXTURE_LINEAR)
                        break;
                    myglobs.TextureQuality = D3DRMTEXTURE_LINEAR;
                    SetRenderState();
                    break;
                }
                /*
                 * Changing the D3D Driver
                 */
                if (LOWORD(wparam) >= MENU_FIRST_DRIVER &&
                    LOWORD(wparam) < MENU_FIRST_DRIVER + MAX_DRIVERS &&
                    myglobs.CurrDriver != LOWORD(wparam) - MENU_FIRST_DRIVER)
		{
                    /*
                     * Release the current viewport and device and create
                     * the new one
                     */
		    int LastDriver = myglobs.CurrDriver;
		    myglobs.bInitialized = FALSE;
                    RELEASE(myglobs.view);
                    RELEASE(myglobs.dev);
                    myglobs.CurrDriver = LOWORD(wparam)-MENU_FIRST_DRIVER;
                    GetClientRect(win, &rc);
                    if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver,
						rc.right, rc.bottom))
		    {
			myglobs.CurrDriver = LastDriver;
			if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver,
						    rc.right, rc.bottom))
			{
			    Msg("Failed to create the D3DRM device.\n%s", D3DRMErrorToString(rval));
			    CleanUpAndPostQuit();
			}
			else
			{
			    Msg("There was not enough video memory available to use the 3D accelerated hardware device.\nRestoring old software device.");
			    myglobs.bInitialized = TRUE;
			}
                    }
		    else
		    {
			myglobs.bInitialized = TRUE;
		    }
                }
                /*
                 * Draw a frame in single step mode after ever command
                 */
                myglobs.bDrawAFrame = TRUE;
            break;
    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;
    case WM_SIZE:
        /*
         * Handle resizing of the window
         */
        {
        int width = LOWORD(lparam);
        int height = HIWORD(lparam);
        if (width && height && myglobs.view && myglobs.dev) {
            int view_width = myglobs.view->GetWidth();
            int view_height = myglobs.view->GetHeight();
            int dev_width = myglobs.dev->GetWidth();
            int dev_height = myglobs.dev->GetHeight();
            /*
             * If the window hasn't changed size and we aren't returning from
             * a minimize, there is nothing to do
             */
            if (view_width == width && view_height == height &&
                !myglobs.bMinimized)
                break;
            if (width <= dev_width && height <= dev_height) {
                /*
                 * If the window has shrunk, we can use the same device with a
                 * new viewport
                 */
				myglobs.bInitialized = FALSE;
                RELEASE(myglobs.view);
                rval = lpD3DRM->CreateViewport(myglobs.dev, myglobs.camera,
                                               0, 0, width, height,
                                               &myglobs.view);
                if (rval != D3DRM_OK) {
                    Msg("Failed to resize the viewport.\n%s",
                        D3DRMErrorToString(rval));
                    CleanUpAndPostQuit();
                    break;
                }
                rval = myglobs.view->SetBack(D3DVAL(5000.0));
                if (rval != D3DRM_OK) {
                    Msg("Failed to set background depth after viewport resize.\n%s",
                        D3DRMErrorToString(rval));
                    CleanUpAndPostQuit();
                    break;
                }
		myglobs.bInitialized = TRUE;
            } else {
                /*
                 * If the window got larger than the current device, create a
                 * new device.
                 */
		myglobs.bInitialized = FALSE;
                RELEASE(myglobs.view);
                RELEASE(myglobs.dev);
                if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver,
					    width, height))
		{
		    myglobs.CurrDriver = 0;
		    if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver,
						width, height))
		    {
			Msg("Failed to create the D3DRM device.\n%s", D3DRMErrorToString(rval));
			CleanUpAndPostQuit();
		    }
		    else
		    {
			Msg("There was not enough video memory available to use the 3D accelerated hardware device.\nUsing software rendering instead.");
			myglobs.bInitialized = TRUE;
		    }
                }
		else
		{
		    myglobs.bInitialized = TRUE;
		}
		myglobs.bInitialized = TRUE;
            }
            /*
             * We must not longer be minimized
             */
            myglobs.bMinimized = FALSE;
        } else {
            /*
             * This is a minimize message
             */
            myglobs.bMinimized = TRUE;
        }
        }
		myglobs.bDrawAFrame = TRUE;
        break;
	case MM_JOY1BUTTONDOWN :
		{

            /* Joystick button pressed. Detect which button, play appropriate
             * sound, then draw bullet hole. To play sound, pass 
             * sndPlaySound() a pointer to an in-memory WAVE file and 
             * specify the SND_MEMORY flag. Specify SND_LOOP and SND_ASYNC
             * so that sound keeps playing until it's turned off after 
             * joystick button is released.
             */
			switch (wparam)
			{
				MessageBox(win,"  .","Button pressed",MB_OK);
			case  JOY_BUTTON1CHG:
            {
                //sndPlaySound(lpSound1, SND_LOOP | SND_ASYNC | SND_MEMORY);
                //DrawHole(hWnd);
				MessageBox(win," 1 .","Button pressed",MB_OK);
				//FireButtonPressed(win);
				break;
            }
            case JOY_BUTTON2CHG:
            {
                //sndPlaySound(lpSound2, SND_LOOP | SND_ASYNC | SND_MEMORY);
                //DrawHole(hWnd);
				break;
            }
    //        break;
			}
			break;
		}
			case MM_JOY1MOVE :
		
			//MessageBox(win," .","Move",MB_OK);
			//myglobs.camera->SetRotation(myglobs.camera,D3DVAL(1),D3DVAL(1),D3DVAL(1),D3DVAL(0.01));
			if (wparam&JOY_BUTTON1) FireButtonPressed(win);
	WORD x, y;
    POINT pt;

    /* Get the current cursor position in screen coordinates.
     */
    //GetCursorPos(&pt);

    /* Joystick positions are expressed in a coordinate system with the
     * origin in the upper left corner and with coordinate ranges from
     * 0 to 65535. Take the 5 most significant bits, so the position is 
     * expressed in the range of 0 to 31.
     */
    x = LOWORD(lparam) >> 11;
    y = HIWORD(lparam) >> 11;

    /* If the joystick is to the left of the center position, then move
     * the cursor position to the left. Otherwise, if the joystick is to 
     * the right of the center position, then move the cursor position 
     * to the right.
     */
if (!klavrot)
{
    if(x <= 12)
	{
		pt.x = pt.x + x - 17;
	myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),-angle);
	}
    else if(x >= 20)
	{
        pt.x = pt.x + x - 15;
		myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),+angle);
	}
	else
	{
		myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(0),D3DVAL(0.00));
	}
}
    /* If the joystick is below the center position, then move the cursor 
     * position down. Otherwise, if the joystick is above the center 
     * position, then move the cursor position up.
     */
    if(y <= 12)
    {
		pt.y = pt.y + y - 17;
	myglobs.camera->SetPosition(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(0.25));
	}
    else if(y >= 20)
	{
		myglobs.camera->SetPosition(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(-0.25));
        pt.y = pt.y + y - 15;
	}
    /* Set the new cursor position.
     */
    //SetCursorPos(pt.x, pt.y);
			break;
		


	case WM_KEYDOWN:
		{
			//face_frame2->SetRotation(myglobs.scene, D3DVAL(0.1),D3DVAL(0.2),D3DVAL(0.3),0.001);
			//face_frame2->SetPosition(myglobs.scene,D3DVAL(1),D3DVAL(0),D3DVAL(0));

			switch (wparam) 
			{
			case VK_DOWN:{
				//myglobs.camera->SetOrientation(myglobs.camera,D3DVAL(0.1),D3DVAL(0.1),D3DVAL(0.1),D3DVAL(0.2),D3DVAL(1),D3DVAL(1));
				//myglobs.camera->SetOrientation(myglobs.camera,D3DVAL(0.1),D3DVAL(0),D3DVAL(0),D3DVAL(0.2),D3DVAL(0),D3DVAL(0));
				//myglobs.camera->SetOrientation(myglobs.camera,D3DVAL(0.1),D3DVAL(0),D3DVAL(0),D3DVAL(0.2),D3DVAL(0),D3DVAL(0));
				//	myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0.1),D3DVAL(0),D3DVAL(0),angle);
				//myglobs.camera->SetPosition(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(1));
					//D3DVECTOR dir;
					//D3DVECTOR up;
					//myglobs.camera->GetOrientation(myglobs.camera,&dir,&up);
					D3DVALUE HERO_STEP=3;
					//D3DVECTOR return_position;
					//D3DVALUE return_theta;
					//myglobs.camera->GetRotation(myglobs.camera,&dir,&return_theta);
					//myglobs.camera->GetPosition(myglobs.camera,&return_position);
										
					//D3DVALUE cx=D3DVAL(return_position.x);//-HERO_STEP*D3DVAL(dir.x);
					//D3DVALUE cz=D3DVAL(return_position.y)-HERO_STEP*D3DVAL(dir.y);
					//D3DVALUE cy=D3DVAL(return_position.z);//-HERO_STEP*D3DVAL(dir.z);
					myglobs.camera->SetPosition(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(-HERO_STEP));
				break;
						 }
			case VK_LEFT:
				{
				//	myglobs.camera->SetPosition(myglobs.camera,D3DVAL(-1),D3DVAL(0),D3DVAL(0));
					myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),-angle);
					klavrot=true;

					
				break;
				}
			case VK_RIGHT:
				{
				//	myglobs.camera->SetPosition(myglobs.camera,D3DVAL(1),D3DVAL(0),D3DVAL(0));
						myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),angle);
						klavrot=true;
				break;
				}
			case VK_UP:
				{
					//myglobs.camera->SetPosition(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(1));
				/*	D3DVECTOR dir;
					D3DVECTOR up;
					myglobs.camera->GetOrientation(myglobs.camera,&dir,&up);
					D3DVALUE HERO_STEP=3;
					D3DVECTOR return_position;
					D3DVALUE return_theta;
					myglobs.camera->GetRotation(myglobs.camera,&dir,&return_theta);
					myglobs.camera->GetPosition(myglobs.camera,&return_position);
										
					D3DVALUE cx=D3DVAL(return_position.x);//+HERO_STEP*D3DVAL(dir.x);
					D3DVALUE cz=D3DVAL(return_position.y)+HERO_STEP*D3DVAL(dir.y);
					D3DVALUE cy=D3DVAL(return_position.z);//+HERO_STEP*D3DVAL(dir.z);
					myglobs.camera->SetPosition(myglobs.camera,D3DVAL(cx),D3DVAL(cy),D3DVAL(cz));
				*/
					int HERO_STEP=3;
					myglobs.camera->SetPosition (myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(HERO_STEP));
				
				


					
				break;
				}
			case VK_TAB:
				{
				 
						myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0.1),D3DVAL(0),D3DVAL(0),-0.0001);
						
										break;
				}
			case VK_INSERT:
				{

					//if (FAILED(lpD3DRM->CreateFrame( myglobs.scene, &artifact_frame))) MessageBox(win,"error","error",MB_OK);
					//if (FAILED(lpD3DRM->CreateMeshBuilder(&artifact_builder))) MessageBox(win,"error","error",MB_OK);
					//if (!build_spaceship(artifact_builder));
					//if (FAILED(artifact_builder->CreateMesh(&artifact_mesh))) MessageBox(win,"error","error",MB_OK);
					//RELEASE(artifact_builder);
					//if (FAILED(artifact_frame->AddVisual((LPDIRECT3DRMVISUAL) artifact_mesh))) MessageBox(win,"error","error",MB_OK);
					//if (FAILED(artifact_frame->SetPosition(myglobs.scene, D3DVAL(2.5), 
                    //                 D3DVAL(1.0), D3DVAL(-13)))) MessageBox(win,"error","error",MB_OK);
					//if (FAILED(artifact_frame->SetOrientation(myglobs.scene, D3DVAL(0.0), D3DVAL(1), D3DVAL(0.0),
                     //                   D3DVAL(2.0), D3DVAL(1.0), -D3DVAL(1.0))));
					//RELEASE(artifact_mesh);
					//RELEASE(artifact_frame);





					break;
				}
			case VK_DELETE:
				{
					FireButtonPressed(win);
					break;
				}

				break;
				}
				break;
				}
	case WM_KEYUP:
		{
				
				//myglobs.camera->SetPosition(myglobs.camera,D3DVAL(-1),D3DVAL(0),D3DVAL(0));
			switch(wparam)
			{
			case VK_LEFT: 
				myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(0),D3DVAL(0.00));
				klavrot=false;
				break;
			case VK_RIGHT:
				myglobs.camera->SetRotation(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(0),D3DVAL(0.00));
				klavrot=false;
				break;


					
			
				case VK_DELETE:
				{
					//myglobs.camera->SetPosition(myglobs.camera,D3DVAL(0),D3DVAL(0),D3DVAL(1));
					//myglobs.camera->Move(D3DVALUE(0.1));
					
				break;
				}
			}
			break;
		}
	case WM_TIMER:
		{
	
			int m;
	
			if (!activated_enemy)
			{
				LPDIRECT3DRMMESHBUILDER builder=NULL;
				LPDIRECT3DRMMESH mesh=NULL;
				LPDIRECT3DRMFRAME frame=NULL;
				builder=NULL; mesh=NULL;frame=NULL;
				lpD3DRM->CreateMeshBuilder(&builder);
				builder->Load("BIGSHIP1.x",NULL,D3DRMLOAD_FROMFILE,NULL,NULL);
				lpD3DRM->CreateFrame(myglobs.scene,	&frame);
				builder->CreateMesh(&mesh);
				frame->SetPosition (myglobs.scene ,D3DVAL(40),D3DVAL(8),D3DVAL(100));
				frame->AddVisual((LPDIRECT3DRMVISUAL)mesh);


				myglobs.camera ->SetPosition(myglobs.scene,D3DVAL(30),D3DVAL(1),D3DVAL(40));

				myglobs.scene->SetSceneBackgroundRGB(D3DVAL(0.2),D3DVAL(0),D3DVAL(0));
							//     
			int k;//float k2=0.1;
			for (k=30;k<100;k=k+2)
			{
				for (int k1=40; k1<100; k1=k1+2)
					{
			//k=30;int k1=40;
							CreateRectangle(k,0,k1);
						//	k2=k2+0.1;
					}
				}
	

			for (m=0; m<MAX_ENEMY; m++)
			{
				activated_enemy=true;
			ENEMIES[m].enemy_frame =NULL;
			ENEMIES[m].enemy_builder =NULL;
			ENEMIES[m].enemy_builder =NULL;
							if (FAILED(lpD3DRM->CreateFrame( myglobs.scene, &(ENEMIES[m].enemy_frame)))) MessageBox(win,"error","error",MB_OK);
					if (FAILED(lpD3DRM->CreateMeshBuilder(&(ENEMIES[m].enemy_builder) ))) MessageBox(win,"error","error",MB_OK);
					if (!build_spaceship(ENEMIES[m].enemy_builder ));
					if (FAILED(ENEMIES[m].enemy_builder ->CreateMesh(&(ENEMIES[m].enemy_mesh) ))) MessageBox(win,"error","error",MB_OK);
					//ENEMIES[m].enemy_builder->SetColorRGB(D3DVAL(200),D3DVAL(0),D3DVAL(0));
					RELEASE(ENEMIES[m].enemy_builder );
					if (FAILED(ENEMIES[m].enemy_frame ->AddVisual((LPDIRECT3DRMVISUAL) ENEMIES[m].enemy_mesh ))) MessageBox(win,"error","error",MB_OK);
					
//					if (FAILED(bullet_frame->SetOrientation(myglobs.scene, D3DVAL(0), D3DVAL(dir.x), D3DVAL(dir.y),
  //                                     D3DVAL(2.0), D3DVAL(1.0), -D3DVAL(1.0))));
					if (FAILED(ENEMIES[m].enemy_frame->SetOrientation(ENEMIES[m].enemy_frame, D3DVAL(0), D3DVAL(1), D3DVAL(0),
                                       D3DVAL(0), D3DVAL(0), D3DVAL(0))));					//bullet_frame->SetRotation(bullet_frame,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),0);
					
					//ENEMIES[m].enemy_frame ->SetMaterialMode(D3DRMMATERIAL_FROMFRAME);
					//ENEMIES[m].enemy_frame ->SetColorRGB(D3DVAL(200),D3DVAL(0),D3DVAL(0));				
					//D3DCOLOR colen;
					//colen=D3DRMCreateColorRGBA(D3DVAL(0.6),D3DVAL(0.6),D3DVAL(0.6),D3DVAL(0.5));
					//ENEMIES[m].enemy_frame ->SetColor(colen);

					//long Rid;
					//ENEMIES[m].enemy_mesh->AddGroup(7*MAX_ENEMY,11*MAX_ENEMY,7,0,&Rid);
					//ENEMIES[m].enemy_mesh->SetGroupColorRGB(Rid,D3DVAL(200),D3DVAL(0),D3DVAL(0));

					

					RELEASE(ENEMIES[m].enemy_mesh );
					
					
					
					ENEMIES[m].x=D3DVAL(2.5+m*4);
					ENEMIES[m].y=-D3DVAL(15);

						if (FAILED(ENEMIES[m].enemy_frame->SetPosition(myglobs.scene,D3DVAL(2.5+m*4), 
                                     D3DVAL(1), -D3DVAL(15)))) MessageBox(win,"error","error",MB_OK);
												if (FAILED(ENEMIES[m].enemy_frame->SetRotation(ENEMIES[m].enemy_frame,D3DVAL(0.01),D3DVAL(0),D3DVAL(0),D3DVAL(0.01))));
			//			RELEASE(bullet_frame);
			
			
			
			
			}
			}
			D3DVECTOR return_position;
			myglobs.camera->GetPosition(myglobs.scene,&return_position);
			if ((!HERO.can_fire)&&(return_position.x<0)&&(return_position.x>-6)&&(return_position.z>95) &&(return_position.z<100))
			{
				HERO.can_fire =true;
				MessageBox(win,"  .    .      Delete     1 .","",MB_OK);
			}
			//HERO.can_fire =true;
			if ((fire)&&(HERO.can_fire) )
			{
				D3DVECTOR dir;
				D3DVECTOR up;
				bullet_frame->GetOrientation(bullet_frame,&dir,&up);
				bullet_frame->SetPosition(bullet_frame,D3DVAL(0),D3DVAL(0.0),D3DVAL(0.5));
				
				light_frame->SetPosition(bullet_frame,D3DVAL(0.25),D3DVAL(0.25),D3DVAL(-0.5));
							D3DVECTOR bullet_position;
			D3DVECTOR enemies_position;
			bullet_frame->GetPosition(myglobs.scene ,&bullet_position);
			const ESIZE=10;
				for (m=0; m<MAX_ENEMY;m++)
				{
				ENEMIES[m].enemy_frame->GetPosition(myglobs.scene ,&enemies_position);
					if ((enemies_position.x+ESIZE>bullet_position.x)&&(enemies_position.x-ESIZE<bullet_position.x)&&(enemies_position.y+ESIZE>bullet_position.y)&&(enemies_position.y-ESIZE<bullet_position.y)&&(enemies_position.z+ESIZE>bullet_position.z)&&(enemies_position.z-ESIZE<bullet_position.z))
					{
						ENEMIES[m].life =0;
					
						ENEMIES[m].y=-500+1*(rand()%1000);
						ENEMIES[m].x=-500+1*(rand()%1000);
						bullet_frame->SetPosition(myglobs.scene,D3DVAL(1000),D3DVAL(1),D3DVAL(1000));
						break;
					}
				}
				
					
					
			}
				const ENEMIES_STEP=0.5;
			//ENEMIES[n].x=ENEMIES[n].x-ENEMIES_STEP;MessageBox(win,"","",MB_OK);
			for(int n=0; n<MAX_ENEMY;n++)
			{
				if (ENEMIES[n].life <=0) 
				{
					ENEMIES[n].life =100;
					ENEMIES[n].enemy_frame ->SetPosition(myglobs.scene,D3DVAL(ENEMIES[n].x),D3DVAL(1),D3DVAL(ENEMIES[n].y));
	
				}
				//ENEMIES[n].x=ENEMIES[n].x-ENEMIES_STEP;MessageBox(win,"","",MB_OK);
				if (ENEMIES[n].life>0) 
				{
					//ENEMIES[n].x=ENEMIES[n].x-ENEMIES_STEP;MessageBox(win,"","",MB_OK);
					D3DVECTOR return_position;
						
						myglobs.camera ->GetPosition(myglobs.scene,&return_position);
						if ((D3DVAL(ENEMIES[n].x)<D3DVAL(return_position.x+1) )&&(D3DVAL(ENEMIES[n].x)>D3DVAL(return_position.x-1))&&(D3DVAL(ENEMIES[n].y)<D3DVAL(return_position.z+1))&&(D3DVAL(ENEMIES[n].y)>D3DVAL(return_position.z-1))) 
						{
							//bool fff;
							HERO.life=100;
							//MessageBox(NULL,"   ,         .","",MB_YESNO);
							  
							//DestroyWindow(win);
							 //char * args[1];
							 //args[0]="end";
							execl("end.exe","",NULL) ;//MessageBox(NULL," ,       ."," error",MB_OK);
							exit(0);
							return 0;
						}
					
				//char string[125];
			//	char string2[125];
				//ltoa(return_position.x,string,10);
				//MessageBox(win,string,string,MB_OK);
					if (return_position.x <D3DVAL(ENEMIES[n].x)) {ENEMIES[n].x=ENEMIES[n].x-D3DVAL(0.25);}
					if (return_position.x >D3DVAL(ENEMIES[n].x)) {ENEMIES[n].x=ENEMIES[n].x+D3DVAL(0.25);}
				//	ltoa(ENEMIES[n].x,string2,10);
				//	strcat(string,"   ");
				//	strcat(string,string2);
				//MessageBox(win,string,string,MB_OK);
					if (return_position.z<ENEMIES[n].y) {ENEMIES[n].y=ENEMIES[n].y-D3DVAL(0.25);}
					if (return_position.z>ENEMIES[n].y) {ENEMIES[n].y=ENEMIES[n].y+D3DVAL(0.25);}
					//ENEMIES[n].enemy_frame ->SetPosition(myglobs.scene,D3DVAL(ENEMIES[n].x),D3DVAL(1),D3DVAL(ENEMIES[n] .y));
					ENEMIES[n].enemy_frame ->SetPosition(myglobs.scene,D3DVAL(ENEMIES[n].x),D3DVAL(1),D3DVAL(ENEMIES[n].y));
					//if (FAILED(ENEMIES[n].enemy_frame->SetOrientation(, D3DVAL(0), D3DVAL(0), D3DVAL(0),
                      //                 D3DVAL(1), D3DVAL(1), D3DVAL(0))));			
					//if (FAILED(ENEMIES[m].enemy_frame->SetOrientation(myglobs.scene, D3DVAL(0), D3DVAL(1), D3DVAL(0),
                      //               D3DVAL(0), D3DVAL(0), D3DVAL(0))));					
					
				}
				else
				{
					//ENEMIES[n].x=rand();
					//ENEMIES[n].y=rand();
					//ENEMIES[n].life =100;
				}
			}

			break;
		}
    case WM_ACTIVATE:
        {
        /*
         * Create a Windows specific D3DRM window device to handle this
         * message
         */
        LPDIRECT3DRMWINDEVICE windev;
        if (!myglobs.dev)
            break;
        if (SUCCEEDED(myglobs.dev->QueryInterface(IID_IDirect3DRMWinDevice,
            (void **) &windev)))  {
                if (FAILED(windev->HandleActivate(wparam)))
                    Msg("Failed to handle WM_ACTIVATE.\n");
                windev->Release();
        } else {
            Msg("Failed to create Windows device to handle WM_ACTIVATE.\n");
        }
        }

					
					//Create_Enemies

		//for(m=0; m<MAX_ENEMY;m++)
		//{
			
					//if (FAILED(bullet_frame->SetOrientation(myglobs.scene, D3DVAL(0.0), D3DVAL(1), D3DVAL(0.0),
                    //                    D3DVAL(2.0), D3DVAL(1.0), -D3DVAL(1.0))));
					//bullet_frame->SetRotation(bullet_frame,D3DVAL(0),D3DVAL(0.1),D3DVAL(0),0);
		//}
			break;
			
        
    case WM_PAINT:
        if (!myglobs.bInitialized || !myglobs.dev)
            return DefWindowProc(win, msg, wparam, lparam);
        /*
         * Create a Windows specific D3DRM window device to handle this
         * message
         */
        RECT r;
        PAINTSTRUCT ps;
        LPDIRECT3DRMWINDEVICE windev;

        if (GetUpdateRect(win, &r, FALSE)) {
            BeginPaint(win, &ps);
            if (SUCCEEDED(myglobs.dev->QueryInterface(IID_IDirect3DRMWinDevice,
                (void **) &windev))) {
                if (FAILED(windev->HandlePaint(ps.hdc)))
                    Msg("Failed to handle WM_PAINT.\n");
                windev->Release();
            } else {
                Msg("Failed to create Windows device to handle WM_PAINT.\n");
            }
            EndPaint(win, &ps);
        }
        break;
    case WM_DISPLAYCHANGE:
	{
	    /*
	     * If this display change message was generated because another application
	     * has gone exclusive, ignore it.
	     */
	    LPDIRECTDRAW lpDD;
	    LPDIRECTDRAWSURFACE lpDDS;
	    DDSURFACEDESC ddsd;
	    HRESULT err;

	    if (DirectDrawCreate(NULL, &lpDD, NULL))
		break;
	    err = lpDD->SetCooperativeLevel(win, DDSCL_NORMAL);
	    if (err) {
		lpDD->Release();
		break;
	    }
	    memset(&ddsd, 0, sizeof(ddsd));
	    ddsd.dwSize = sizeof(ddsd);
	    ddsd.dwFlags = DDSD_CAPS;
	    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
	    err = lpDD->CreateSurface(&ddsd, &lpDDS, NULL);
	    if (err == DDERR_NOEXCLUSIVEMODE) {
		// This exclusive mode generated WM_DISPLAYCHANGE, ignoring
		lpDD->Release();
		break;
	    }
	    if (!err)
		lpDDS->Release();
	    lpDD->Release();	    
	}
	GetClientRect(win, &rc);
	myglobs.bInitialized = FALSE;
        if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver,
				    rc.right, rc.bottom)) {
	    myglobs.CurrDriver = 0;
	    if (rval = CreateDevAndView(lpDDClipper, myglobs.CurrDriver,
					rc.right, rc.bottom))
	    {
		Msg("Failed to create the D3DRM device.\n%s", D3DRMErrorToString(rval));
		CleanUpAndPostQuit();
	    }
	    else
	    {
		// Don't bother the user with an error message here
		//Msg("There was not enough video memory available to use the 3D accelerated hardware device.\nUsing software rendering instead.");
		myglobs.bInitialized = TRUE;
	    }
        }
	else
	{
	    myglobs.bInitialized = TRUE;
	}
	break;
    default:
        return DefWindowProc(win, msg, wparam, lparam);
    }
    return 0L;
}

/*
 * SetRenderState
 * Set the render quality, dither toggle and shade info if any of them has
 * changed
 */
HRESULT
SetRenderState(void)
{
    HRESULT rval;
    /*
     * Set the render quality (light toggle, fill mode, shade mode)
     */
    if (myglobs.dev->GetQuality() != myglobs.RenderQuality) {
        rval = myglobs.dev->SetQuality(myglobs.RenderQuality);
        if (rval != D3DRM_OK)
	    return rval;
    }
    /*
     * Set dithering toggle
     */
    if (myglobs.dev->GetDither() != myglobs.bDithering) {
        rval = myglobs.dev->SetDither(myglobs.bDithering);
        if (rval != D3DRM_OK) 
	    return rval;
    }
    /*
     * Set the texture quality (point or linear filtering)
     */
    if (myglobs.dev->GetTextureQuality() != myglobs.TextureQuality) {
        rval = myglobs.dev->SetTextureQuality(myglobs.TextureQuality);
        if (rval != D3DRM_OK) 
	    return rval;
    }
    /*
     * Set shade info based on current bits per pixel
     */
    switch (myglobs.BPP) {
    case 1:
        if (rval = myglobs.dev->SetShades(4))
            return rval;
        if (rval = lpD3DRM->SetDefaultTextureShades(4))
            return rval;
        break;
    case 16:
        if (rval = myglobs.dev->SetShades(32))
            return rval;
        if (rval = lpD3DRM->SetDefaultTextureColors(64))
            return rval;
        if (rval = lpD3DRM->SetDefaultTextureShades(32))
            return rval;
        break;
    case 24:
    case 32:
        if (rval = myglobs.dev->SetShades(256))
            return rval;
        if (rval = lpD3DRM->SetDefaultTextureColors(64))
            return rval;
        if (rval = lpD3DRM->SetDefaultTextureShades(256))
            return rval;
        break;
    }
    return D3DRM_OK;
}

/****************************************************************************/
/*                          Additional Functions                            */
/****************************************************************************/
/*
 * ReadMouse
 * Returns the mouse status for interaction with sample code
 */
void
ReadMouse(int* b, int* x, int* y)
{
    *b = myglobs.mouse_buttons;
    *x = myglobs.mouse_x;
    *y = myglobs.mouse_y;
}

/*
 * InitGlobals
 * Initialize the global variables
 */
void
InitGlobals(void)
{
    lpD3DRM = NULL;
    memset(&myglobs, 0, sizeof(myglobs));
    myglobs.RenderQuality = D3DRMLIGHT_ON | D3DRMFILL_SOLID |
                            D3DRMSHADE_GOURAUD;
    myglobs.TextureQuality = D3DRMTEXTURE_NEAREST;
}

/*
 * CleanUpAndPostQuit
 * Release all D3DRM objects, post a quit message and set the bQuit flag
 */
void
CleanUpAndPostQuit(void)
{
    myglobs.bInitialized = FALSE;
    RELEASE(myglobs.scene);
    RELEASE(myglobs.camera);
    RELEASE(myglobs.view);
    RELEASE(myglobs.dev);
    RELEASE(lpD3DRM);
    RELEASE(lpDDClipper);
    myglobs.bQuit = TRUE;
}
